!
! Daily update of leaf area index etc.
!
! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/stomate.f90,v 1.15.2.2 2004/01/13 10:59:45 ssipsl Exp $
!
MODULE stomate

  ! modules used:

  USE ioipsl
  USE reqdprec
  USE constantes
  USE constantes_veg
  USE constantes_co2
  USE stomate_netcdf
  USE stomate_constants
  USE stomate_io
  USE stomate_data
  USE stomate_natagritot
  USE stomate_season
  USE stomate_lpj
  USE stomate_assimtemp
  USE stomate_litter
  USE stomate_vmax
  USE stomate_soilcarbon

  IMPLICIT NONE

  INCLUDE "netcdf.inc"

  PRIVATE
  PUBLIC stomate_soil,stomate_main,stomate_clear,pondere_nat,pondere_nat_vegfrac,pondere_vegfrac

  !
  ! variables used inside stomate module : declaration and initialisation
  !

  ! total natural space (fraction of total space)
    REAL(stnd), ALLOCATABLE, SAVE, DIMENSION(:,:)                 :: veget_max
  ! total natural space (fraction of total space)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: space_nat
  ! carbon pool: active, slow, or passive, natural and agricultural (gC/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: carbon
  ! density of individuals (1/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ind
  ! daily moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_daily
  ! daily litter humidity
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: litterhum_daily
  ! daily 2 meter temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_daily
  ! daily minimum 2 meter temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_min_daily
  ! daily surface temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: tsurf_daily
  ! daily soil temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_daily
  ! daily soil humidity
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_daily
  ! daily precipitations (mm/day) (for phenology)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_daily
  ! daily gross primary productivity (gC/(m**2 of nat/agri ground)/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_daily
  ! daily net primary productivity (gC/(m**2 of nat/agri ground)/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_daily
  ! Turnover rates (gC/(m**2 of nat/agri ground)/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: turnover_daily
  ! Probability of fire
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: fireindex
  ! Longer term total litter above the ground (gC/m**2 of nat/agri ground)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: firelitter
  ! "monthly" moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_month
  ! "weekly" moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_week
  ! "long term" 2 meter temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_longterm
  ! "long term" 2 meter reference temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: tlong_ref
  ! "monthly" 2 meter temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_month
  ! "weekly" 2 meter temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_week
  ! "monthly" soil temperatures (K)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_month
  ! "monthly" soil humidity
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_month
  ! growing degree days, threshold -5 deg C (for phenology)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_m5_dormance
  ! growing degree days, since midwinter (for phenology)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_midwinter
  ! number of chilling days since leaves were lost (for phenology)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ncd_dormance
  ! number of growing days, threshold -5 deg C (for phenology)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ngd_minus5
  ! last year's maximum moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_lastyear
  ! this year's maximum moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_thisyear
  ! last year's minimum moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_lastyear
  ! this year's minimum moisture availability
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_thisyear
  ! last year's maximum "weekly" GPP (gC/m**2 covered/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_lastyear
  ! this year's maximum "weekly" GPP (gC/m**2 covered/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_thisyear
  ! last year's annual GDD0
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_lastyear
  ! this year's annual GDD0
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_thisyear
  ! last year's annual precipitation (mm/year)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_lastyear
  ! this year's annual precipitation (mm/year)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_thisyear
  ! PFT exists (equivalent to veget > 0 for natural PFTs)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)       :: PFTpresent
  ! "long term" net primary productivity (gC/(m**2 of nat/agri ground)/year)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_longterm
  ! last year's maximum leaf mass, for each PFT (gC/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_lastyearmax
  ! this year's maximum leaf mass, for each PFT (gC/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_thisyearmax
  ! last year's maximum fpc for each natural PFT, on *natural* ground
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_lastyear
  ! this year's maximum fpc for each PFT, on *total* ground (see stomate_season)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_thisyear
  ! "long term" turnover rate (gC/(m**2 of nat/agri ground)/year)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: turnover_longterm
  ! "weekly" GPP (gC/day/(m**2 covered)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_week
  ! biomass (gC/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: biomass
  ! is the plant senescent? (only for deciduous trees - carbohydrate reserve)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)       :: senescence
  ! how many days ago was the beginning of the growing season
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: when_growthinit
  ! age (years). For trees, mean stand age. For grasses, ears since introduction of PFT
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: age
  ! Winter too cold? between 0 and 1
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: adapted
  ! Winter sufficiently cold? between 0 and 1
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: regenerate
  ! fraction of litter above the ground belonging to different PFTs,
  !   separated for natural and agricultural PFTs
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: litterpart
  ! metabolic and structural litter, natural and agricultural, above and below ground
  !   (gC/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: litter
  ! dead leaves on ground, per PFT, metabolic and structural, in gC/(m**2 of nat/agri ground)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: dead_leaves
  ! black carbon on the ground (gC/(m**2 of total ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: black_carbon
  ! ratio Lignine/Carbon in structural litter, above and below ground, natural and agricultural
  !   (gC/(m**2 of nat/agri ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: lignin_struc
  ! carbon emitted into the atmosphere by fire (living and dead biomass)
  !   (in gC/m**2 of average ground/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: co2_fire
  ! co2 taken up (gC/(m**2 of total ground)/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: co2_to_bm_dgvm
  ! heterotrophic respiration (gC/day/m**2 of total ground)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero
  ! maintenance respiration (gC/day/(m**2 of total ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_maint
  ! growth respiration (gC/day/(m**2 of total ground))
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_growth
  ! vegetation fractions (on natural/agri ground) after last light competition
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_lastlight
  ! is the PFT everywhere in the grid box or very localized (after its intoduction)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: everywhere
  ! in order for this PFT to be introduced, does it have to be present in an adjacent grid box?
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)       :: need_adjacent
  ! leaf age (d)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: leaf_age
  ! fraction of leaves in leaf age class
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: leaf_frac
  ! How much time ago was the PFT eliminated for the last time (y)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: RIP_time
  ! duration of dormance (d)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_lowgpp
  ! time elapsed since strongest moisture availability (d)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_hum_min
  ! minimum moisture during dormance
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: hum_min_dormance
  ! factor that allows to transform "veget_max_x" (Sechiba) into "veget_max" (Stomate).
  ! veget_max_x takes into account the ice fraction, while veget_max is defined on ice_free
  ! land surface only. Moreover, PFTs between Sechiba and Stomate are not necessarily identical.
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: fvm
  ! factor that allows to transform "veget_x" (Sechiba) into "veget" (Stomate) [see "fvm"].
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: fv
  ! time constant of probability of a leaf to be eaten by a herbivore (days)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: herbivores

  ! moisture control of heterotrophic respiration
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: control_moist
  ! temperature control of heterotrophic respiration, above and below
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: control_temp
  ! quantity of carbon going into carbon pools from litter decomposition
  !   (gC/(m**2 of nat/agri ground)/day)
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: soilcarbon_input
  ! times at which soil forcing file is written
  REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)      :: times
  ! how many states were calculated for a given soil forcing time step
    ! turnover time of leaves
    REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:,:)                           :: turnover_time
    ! daily total CO2 flux (gC/m**2/day)
    REAL(stnd),ALLOCATABLE,SAVE,DIMENSION(:)                           :: co2_flux_daily 

  INTEGER(long),ALLOCATABLE,SAVE,DIMENSION(:)   :: nforce

  !forcing data in memory
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:)         :: clay_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: humrel_daily_x_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:)         :: litterhum_daily_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:)         :: t2m_daily_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:)         :: t2m_min_daily_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:)         :: tsurf_daily_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: tsoil_daily_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: soilhum_daily_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:)         :: precip_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: gpp_daily_x_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_x_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: veget_max_x_fm
  REAL(stnd),ALLOCATABLE,DIMENSION(:,:,:)       :: lai_x_fm
  INTEGER(long),ALLOCATABLE,SAVE,DIMENSION(:)   :: isf
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:)         :: nf_written
  INTEGER(long),ALLOCATABLE,SAVE,DIMENSION(:)   :: nf_cumul
    ! first call
    LOGICAL, SAVE                                      :: l_first_stomate = .TRUE.
    ! flag for cumul of forcing if teststomate
    LOGICAL, SAVE                                      :: cumul_forcing=.FALSE.
CONTAINS

  SUBROUTINE stomate_main (kjit, kjpij, kjpindex, dtradia, dt_slow, qsintcst, &
                           ldrestart_read, ldrestart_write, ldforcing_write, ldcarbon_write, &
                           index, lalo, neighbours, resolution, fraction_nobio, clay, &
                           t2m, t2m_min, temp_sol, stempdiag, &
                           humrel_x, shumdiag, litterhumdiag, precip_rain, precip_snow, gpp_x, &
                           deadleaf_cover, &
                           assim_param_x, &
                           lai_x, height_x, veget_x, veget_max_x, qsintmax, &
                           hist_id, rest_id_stom, hist_id_stom, &
                           co2_flux)

    ! 0 interface description
    !
    ! 0.1 input
    !
    ! 0.1.1 input scalar
    !
    ! Time step number
    INTEGER(long), INTENT(in)                          :: kjit
    ! Domain size
    INTEGER(long),INTENT (in)                          :: kjpindex
    ! Total size of the un-compressed grid
    INTEGER(long), INTENT(in)                          :: kjpij
    ! Time step of SECHIBA
    REAL(stnd),INTENT (in)                             :: dtradia
    ! Time step of STOMATE
    REAL(stnd),INTENT (in)                             :: dt_slow
    ! Logical for _restart_ file to read
    REAL(stnd),INTENT (in)                             :: qsintcst
    ! Interception reservoir coefficient
    LOGICAL, INTENT(in)                                :: ldrestart_read
    ! Logical for _restart_ file to write
    LOGICAL, INTENT(in)                                :: ldrestart_write
    ! Logical for _forcing_ file to write
    LOGICAL, INTENT(in)                                :: ldforcing_write
    ! Logical for _carbon_forcing_ file to write
    LOGICAL, INTENT(in)                                :: ldcarbon_write
    ! _Restart_ file and _history_ file identifier
    INTEGER(long),INTENT (in)                          :: hist_id
    ! STOMATE's _Restart_ file file identifier
    INTEGER(long),INTENT (in)                          :: rest_id_stom
    ! STOMATE's _history_ file file identifier
    INTEGER(long),INTENT (in)                          :: hist_id_stom
    !
    ! 0.1.2 input fields
    !
    ! Indeces of the points on the map
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)    :: index
    ! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (in)     :: lalo
    ! neighoring grid points if land
    INTEGER(long), DIMENSION (kjpindex,8), INTENT(in)  :: neighbours
    ! size in x an y of the grid (m)
    REAL(stnd), DIMENSION (kjpindex,2), INTENT(in)     :: resolution
    ! fraction of grid cell covered by lakes, land ice, cities, ...
    REAL(stnd), DIMENSION (kjpindex), INTENT(in)       :: fraction_nobio
    ! clay fraction
    REAL(stnd),DIMENSION(kjpindex), INTENT(in)         :: clay
    ! Relative humidity ("moisture availability")
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT (in)  :: humrel_x
    ! 2 m air temperature (K)
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)        :: t2m
    ! min. 2 m air temp. during forcing time step (K)
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)        :: t2m_min
    ! Surface temperature
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: temp_sol
    ! Soil temperature
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (in)  :: stempdiag
    ! Relative soil moisture
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (in)  :: shumdiag
    ! Litter humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: litterhumdiag
    ! Rain precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: precip_rain
    ! Snow precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: precip_snow
    ! GPP (gC/(m**2 of total ground)/time step)
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in)    :: gpp_x
    !
    ! 0.2 output
    !
    ! 0.2.1 output scalar
    !
    ! 0.2.2 output fields
    !
    ! CO2 flux in gC/m**2 of average ground/second
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)      :: co2_flux
    !
    ! 0.3 modified
    !
    ! 0.3.1 modified scalar
    ! 0.3.2 modified fields
    !
    ! Surface foliere
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: lai_x
    ! Fraction of vegetation type from hydrological module. Takes into account ice etc.
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: veget_x
    ! Maximum fraction of vegetation type from hydrological module. Takes into account ice etc.
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: veget_max_x
    ! height (m)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: height_x
    ! min+max+opt temps & vmax for photosynthesis
    REAL(stnd),DIMENSION (kjpindex,nvm,npco2), INTENT (inout) :: assim_param_x
    ! fraction of soil covered by dead leaves
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)    :: deadleaf_cover
    ! Maximum water on vegetation for interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: qsintmax
    !
    ! 0.4 local declaration
    !

    ! 0.4.1 variables defined on nvm in SECHIBA, npft in STOMATE/LPJ

    ! moisture availability
    REAL(stnd), DIMENSION(kjpindex,npft)               :: humrel
    ! gross primary productivity (gC/(m**2 of total ground)/day)
    REAL(stnd), DIMENSION(kjpindex,npft)               :: gpp
    ! fractional coverage: actually covered space, taking into account LAI (= grid scale fpc).
    !   Fraction of nat/agri ground.
    REAL(stnd), DIMENSION(kjpindex,npft)               :: veget

    ! 0.4.2 other

    ! time step of STOMATE in days
    REAL(stnd), SAVE                                   :: dt_days
    ! to check
    REAL(stnd), SAVE                                   :: day_counter
    ! date (d)
    INTEGER(long), SAVE                                :: date
    ! soil level used for LAI
    INTEGER(short), SAVE                               :: lcanop
    ! STOMATE time step read in restart file
    REAL(stnd)                                         :: dt_days_read
    ! Maximum STOMATE time step (days)
    REAL(stnd), PARAMETER                              :: max_dt_days = 5.
    ! is it time for Stomate or update of LAI etc.?
    LOGICAL                                            :: do_slow
    ! Do update of yearly variables? This variable must be .TRUE. once a year
    LOGICAL                                            :: EndOfYear
    ! Writing frequency for history file (d)
    REAL(stnd)                                         :: hist_days
    ! precipitation (mm/day)
    REAL(stnd), DIMENSION(kjpindex)                    :: precip
    ! Maximum rate of carboxylation
    REAL(stnd), DIMENSION(kjpindex,npft)               :: vcmax
    ! Maximum rate of RUbp regeneration
    REAL(stnd), DIMENSION(kjpindex,npft)               :: vjmax
    ! Min temperature for photosynthesis (deg C)
    REAL(stnd), DIMENSION(kjpindex,npft)               :: t_photo_min
    ! Opt temperature for photosynthesis (deg C)
    REAL(stnd), DIMENSION(kjpindex,npft)               :: t_photo_opt
    ! Max temperature for photosynthesis (deg C)
    REAL(stnd), DIMENSION(kjpindex,npft)               :: t_photo_max
    ! leaf area index
    REAL(stnd), DIMENSION(kjpindex,npft)               :: lai
    ! total autotrophic respiration (gC/day/(m**2 of total ground))
    REAL(stnd), DIMENSION(kjpindex)                    :: resp_auto_tot
    ! total photosynthesis (gC/day/(m**2 of total ground))
    REAL(stnd), DIMENSION(kjpindex)                    :: gpp_tot
    ! for forcing file: "daily" moisture availability
    REAL(stnd), DIMENSION(kjpindex,nvm)                :: humrel_daily_x
    ! for forcing file: "daily" gpp
    REAL(stnd), DIMENSION(kjpindex,nvm)                :: gpp_daily_x
    ! total "vegetation" cover
    REAL(stnd), DIMENSION(kjpindex)                    :: cvegtot
    ! height of vegetation (m)
    REAL(stnd), DIMENSION(kjpindex,npft)               :: height
    INTEGER(short)                                     :: ji, jv, i, j
    REAL (stnd)                                        :: trans_veg
    REAL(stnd)                                         :: tmp_day(1)
    ! To store variables names for I/O
    CHARACTER*80                                       :: var_name
    INTEGER(long)                                      :: ier
    ! moisture control of heterotrophic respiration
    REAL(stnd),DIMENSION(kjpindex,nlevs)               :: control_moist_inst
    ! temperature control of heterotrophic respiration, above and below
    REAL(stnd),DIMENSION(kjpindex,nlevs)               :: control_temp_inst
    ! quantity of carbon going into carbon pools from litter decomposition
    !   (gC/(m**2 of nat/agri ground)/day)
    REAL(stnd),DIMENSION(kjpindex,ncarb,nvegtypes)     :: soilcarbon_input_inst
    ! time step of soil forcing file (in days)
    REAL(stnd), SAVE                                   :: dt_forcesoil
    INTEGER, SAVE                                      :: nparan
    INTEGER, PARAMETER                                 :: nparanmax=36
    REAL(stnd)                                         :: sf_time
    INTEGER, SAVE                                      :: iatt=1
    REAL(stnd)                                         :: ein_jahr, ein_tag
    INTEGER(long)                                      :: max_totsize, totsize_1step
    REAL(stnd)                                         :: xn
    INTEGER(long), SAVE                                :: nsfm, nsft
    INTEGER(long), SAVE                                :: iisf
    CHARACTER(LEN=100)                                 :: forcing_name, Cforcing_name, varname
    INTEGER, SAVE                                      :: Cforcing_id
    INTEGER                                            :: vid
    INTEGER, PARAMETER                                 :: ndm = 10
    INTEGER, DIMENSION(ndm)                            :: idim
    INTEGER                                            :: ndim
    INTEGER                                            :: numdim,nneigh,direct
    INTEGER(long)                                      :: year, month, day   
    REAL(stnd)                                         :: sec

    ! first of all: store time step in common value
    itime = kjit
!!!
!!! Jan : Preparation for computing the dates here with the IOIPSL calendar
!!!
!!!    CALL itau2ymds(itime, dtradia, year, month, day, sec)
!!!    WRITE(*,*) 'DATE ymds', year, month, day, sec, '-- stp --', itime

! Height of vegetation must be initialized to height_x at each time step 
    DO j=1,npft
       height(:,j)=height_x(:,j+1)
    END DO

    !
    ! 1 do initialisation
    !

    IF (l_first_stomate) THEN

      IF (long_print) WRITE (numout,*) ' l_first_stomate : call stomate_init '

      !
      ! 1.1 allocation, file definitions. Set flags.
      !

      CALL stomate_init (kjpij, kjpindex, index, ldforcing_write, lalo, &
                          rest_id_stom, hist_id_stom)
 
      !
      ! 1.2 read PFT data
      !

      CALL data (kjpindex, lalo)
 
      !
      ! 1.3 read STOMATE's start file
      !

      CALL readstart (kjpindex, index, lalo, resolution, &
                      day_counter, dt_days_read, date, &
                      ind, adapted, regenerate, &
                      humrel_daily, litterhum_daily, &
                      t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
                      soilhum_daily, precip_daily, &
                      gpp_daily, npp_daily, turnover_daily, &
                      humrel_month, humrel_week, t2m_longterm, tlong_ref, t2m_month, t2m_week, &
                      tsoil_month, soilhum_month, fireindex, firelitter, &
                      maxhumrel_lastyear, maxhumrel_thisyear, &
                      minhumrel_lastyear, minhumrel_thisyear, &
                      maxgppweek_lastyear, maxgppweek_thisyear, &
                      gdd0_lastyear, gdd0_thisyear, &
                      precip_lastyear, precip_thisyear, &
                      gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
                      PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
                      maxfpc_lastyear, maxfpc_thisyear, &
                      turnover_longterm, gpp_week, biomass, &
                      fvm, fv, leaf_age, leaf_frac, &
                      senescence, when_growthinit, age, &
                      resp_hetero, resp_maint, resp_growth, co2_fire, co2_to_bm_dgvm, &
                      veget_lastlight, everywhere, need_adjacent, RIP_time, time_lowgpp, &
                      time_hum_min, hum_min_dormance, &
                      litterpart, litter, dead_leaves, &
                      carbon, black_carbon, lignin_struc,turnover_time)
          !
      ! 1.4 read the boundary conditions
      !

      CALL readbc (kjpindex, lalo, resolution, tlong_ref)

      !
      ! 1.5 check time step
      !

      ! 1.5.1 allow STOMATE's time step to change although this is dangerous

      dt_days = dt_slow / one_day

      IF ( dt_days .NE. dt_days_read ) THEN
        WRITE(numout,*) 'slow_processes: STOMATE time step changes:', &
                         dt_days_read,' -> ',dt_days
      ENDIF

      ! 1.5.2 time step has to be a multiple of a full day

      IF ( ( dt_days - REAL(NINT(dt_days)) ) .GT. min_stomate ) THEN
        WRITE(numout,*) 'slow_processes: STOMATE time step is wrong:', dt_days,' days.'
        STOP
      ENDIF

      ! 1.5.3 upper limit to STOMATE's time step

      IF ( dt_days .GT. max_dt_days ) THEN
        WRITE(numout,*) 'slow_processes: STOMATE time step too large:', dt_days,' days.'
        STOP
      ENDIF

      ! 1.5.4 STOMATE time step must not be less than the forcing time step

      IF ( dtradia .GT. dt_days * one_day ) THEN
        WRITE(numout,*) 'slow_processes: STOMATE time step smaller than forcing time step.'
        STOP
      ENDIF

      ! 1.5.5 some more messages

      WRITE(numout,*) 'slow_processes, STOMATE time step (d): ', dt_days

      !
      ! 1.6 write forcing file for stomate?
      !

      IF ( ldforcing_write ) THEN

        WRITE(numout,*) 'writing a forcing file for STOMATE.'

        CALL ioget_calendar(ein_jahr, ein_tag)

        !Config  Key  = STOMATE_FORCING_MEMSIZE
        !Config  Desc = Size of STOMATE forcing data in memory (MB)
        !Config  Def  = 50
        !Config  Help = This variable determines how many forcing states will
        !Config         be kept in memory. Must be a compromise between memory
        !Config         use and frequeny of disk access.

        max_totsize = 50
        CALL getin('STOMATE_FORCING_MEMSIZE', max_totsize)
        max_totsize = max_totsize * 1000000
 
        totsize_1step = SIZE(clay)*KIND(clay) + &
                        SIZE(humrel_daily_x)*KIND(humrel_daily_x) + &
                        SIZE(litterhum_daily)*KIND(litterhum_daily) + &
                        SIZE(t2m_daily)*KIND(t2m_daily) + &
                        SIZE(t2m_min_daily)*KIND(t2m_min_daily) + &
                        SIZE(tsurf_daily)*KIND(tsurf_daily) + &
                        SIZE(tsoil_daily)*KIND(tsoil_daily) + &
                        SIZE(soilhum_daily)*KIND(soilhum_daily) + &
                        SIZE(precip_daily)*KIND(precip_daily) + &
                        SIZE(gpp_daily_x)*KIND(gpp_daily_x) + &
                        SIZE(veget_x)*KIND(veget_x) + &
                        SIZE(veget_max_x)*KIND(veget_max_x) + &
                        SIZE(lai_x)*KIND(lai_x)
 
        ! total number of forcing steps
        nsft =  INT(ein_jahr/(dt_slow/ein_tag))
 
        ! number of forcing steps in memory
        nsfm = MIN( nsft, MAX( 1, NINT(FLOAT(max_totsize)/FLOAT(totsize_1step)) ) )

        CALL init_forcing (kjpindex,nsfm,nsft)

        isf(:) = (/ (i,i=1,nsfm) /)
        nf_written(:) = .FALSE.
        nf_cumul(:) = 0

        iisf = 0

        !Config  Key  = STOMATE_FORCING_NAME
        !Config  Desc = Name of STOMATE's forcing file
        !Config  Def  = stomate_forcing.nc
        !Config  Help = Name that will be given to STOMATE's offline forcing file
 
        forcing_name = stomate_forcing_name
        CALL getin('STOMATE_FORCING_NAME', forcing_name)

        CALL SYSTEM( 'rm -f '//TRIM(forcing_name) )
        ier = NF_CREATE( TRIM(forcing_name), NF_SHARE, forcing_id )

        ier = NF_PUT_ATT_STND( forcing_id, NF_GLOBAL, 'dtradia', dtradia )
        ier = NF_PUT_ATT_STND( forcing_id, NF_GLOBAL, 'dt_slow', dt_slow )
        ier = NF_PUT_ATT_STND( forcing_id, NF_GLOBAL, 'nsft', REAL(nsft,stnd) )
        ier = NF_PUT_ATT_STND( forcing_id, NF_GLOBAL, 'kjpij', REAL(kjpij,stnd) )
        ier = NF_PUT_ATT_STND( forcing_id, NF_GLOBAL, 'kjpindex', REAL(kjpindex,stnd) )
        numdim=1
        ier = NF_DEF_DIM( forcing_id, 'points', kjpindex, numdim )
        numdim=2
        ier = NF_DEF_DIM( forcing_id, 'layers', nbdl, numdim )
        numdim=3
        ier = NF_DEF_DIM( forcing_id, 'pft', nvm, numdim )
        numdim=4
        direct=2
        ier = NF_DEF_DIM( forcing_id, 'direction', direct, numdim )
        numdim=5
        nneigh=8
        ier = NF_DEF_DIM( forcing_id, 'nneigh', nneigh, numdim )
        numdim=6
        ier = NF_DEF_DIM( forcing_id, 'time', nsft, numdim)

        ndim = 1 ; idim(1) = 1 ; ier = NF_DEF_VAR_STND( forcing_id, 'points', ndim, idim, vid )
        ndim = 1 ; idim(1) = 2 ; ier = NF_DEF_VAR_STND( forcing_id, 'layers', ndim, idim, vid )
        ndim = 1 ; idim(1) = 3 ; ier = NF_DEF_VAR_STND( forcing_id, 'pft', ndim, idim, vid )
        ndim = 1 ; idim(1) = 4 ; ier = NF_DEF_VAR_STND( forcing_id, 'direction', ndim, idim, vid )
        ndim = 1 ; idim(1) = 5 ; ier = NF_DEF_VAR_STND( forcing_id, 'nneigh', ndim, idim, vid )
        ndim = 1 ; idim(1) = 6 ; ier = NF_DEF_VAR_STND( forcing_id, 'time', ndim, idim, vid )

        ndim = 1 ; idim(1) = 1 ; varname = 'index'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2 ; idim(1) = 1 ; idim(2) = 4 ; varname = 'lalo'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2 ; idim(1) = 1 ; idim(2) = 5 ; varname = 'neighbours'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2 ; idim(1) = 1 ; idim(2) = 4 ; varname = 'resolution'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 2; idim(1) = 1 ; idim(2) = 6 ; varname = 'clay'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 3 ; idim(ndim) = 6 ; varname = 'humrel'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2; idim(1) = 1 ; idim(2) = 6 ; varname = 'litterhum'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2; idim(1) = 1 ; idim(2) = 6 ; varname = 't2m'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2; idim(1) = 1 ; idim(2) = 6 ; varname = 't2m_min'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2; idim(1) = 1 ; idim(2) = 6 ; varname = 'tsurf'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 2 ; idim(ndim) = 6 ; varname = 'tsoil'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 2 ; idim(ndim) = 6 ; varname = 'soilhum'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 2; idim(1) = 1 ; idim(2) = 6 ; varname = 'precip'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 3 ; idim(ndim) = 6 ; varname = 'gpp'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 3 ; idim(ndim) = 6 ; varname = 'veget'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 3 ; idim(ndim) = 6 ; varname = 'veget_max'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )
        ndim = 3; idim(1) = 1 ; idim(2) = 3 ; idim(ndim) = 6 ; varname = 'lai'
        ier = NF_DEF_VAR_STND( forcing_id, TRIM(varname), ndim, idim, vid )

        ier = NF_ENDDEF( forcing_id )

        ier = NF_PUT_VAR_STND( forcing_id, 'points', (/ (REAL(i,stnd),i=1,kjpindex) /) )
        ier = NF_PUT_VAR_STND( forcing_id, 'layers', (/ (REAL(i,stnd),i=1,nbdl) /) )
        ier = NF_PUT_VAR_STND( forcing_id, 'pft', (/ (REAL(i,stnd),i=1,nvm) /) )
        ier = NF_PUT_VAR_STND( forcing_id, 'direction', (/ (REAL(i,stnd),i=1,2) /) )
        ier = NF_PUT_VAR_STND( forcing_id, 'nneigh', (/ (REAL(i,stnd),i=1,8) /) )
        ier = NF_PUT_VAR_STND( forcing_id, 'time', (/ (REAL(i,stnd),i=1,nsft) /) )

        ier = NF_PUT_VAR_STND( forcing_id, 'index', REAL(index,stnd) )
        ier = NF_PUT_VAR_STND( forcing_id, 'lalo', lalo )
        ier = NF_PUT_VAR_STND( forcing_id, 'neighbours', REAL(neighbours,stnd) )
        ier = NF_PUT_VAR_STND( forcing_id, 'resolution', resolution )

      ENDIF

      !
      ! 1.7 write forcing file for the soil?
      !

      IF ( ldcarbon_write ) THEN

        !
        ! time step of forcesoil
        !
  
        !Config  Key  = FORCESOIL_STEP_PER_YEAR
        !Config  Desc = Number of time steps per year for carbon spinup
        !Config  Def  = 12
        !Config  Help = Number of time steps per year for carbon spinup
  
        nparan = 12
        CALL getin('FORCESOIL_STEP_PER_YEAR', nparan)
        IF ( nparan .LT. 1 ) nparan = 1
  
        dt_forcesoil = 0.
        nparan = nparan+1
        DO WHILE ( dt_forcesoil .LT. dt_slow/un_jour )
          nparan = nparan-1
          IF ( nparan .LT. 1 ) STOP 'Problem 1 with number of soil forcing time steps.'
          dt_forcesoil = one_year / FLOAT(nparan)
        ENDDO
  
        IF ( nparan .GT. nparanmax ) STOP 'Problem 2 with number of soil forcing time steps.'

        WRITE(*,*) 'time step of soil forcing (d): ',dt_forcesoil

        ALLOCATE( times(0:nparan))
        times(:) = (/ ( (FLOAT(i)*dt_forcesoil),i=0,nparan ) /)

        ALLOCATE( nforce(nparan))
        nforce(:) = 0

        ALLOCATE (control_moist(kjpindex,nlevs,nparan))
        ALLOCATE (control_temp(kjpindex,nlevs,nparan))
        ALLOCATE (soilcarbon_input(kjpindex,ncarb,nvegtypes,nparan))

        control_moist(:,:,:) = zero
        control_temp(:,:,:) = zero
        soilcarbon_input(:,:,:,:) = zero

      ENDIF

      !
      ! 1.8 calculate STOMATE's vegetation fractions from veget_x, veget_max_x, fv, and fvm
      !

      CALL stomate_vegconvert (kjpindex,'in',fraction_nobio, &
                                 veget_max_x,veget_x,veget_max,veget,fvm,fv)

      !
      ! 1.9 initialize some variables
      !     STOMATE diagnoses some variables for SECHIBA: assim_param, deadleaf_cover, etc.
      !     These variables can be recalculated easily from STOMATE's prognostic variables.
      !     height is saved in Sechiba.
      !

      IF ( control%ok_stomate ) THEN
        CALL stomate_var_init (kjpindex, qsintcst, &
                               veget, veget_max, leaf_age, leaf_frac, tlong_ref, &
                               t2m_month, dead_leaves, &
                               veget_x, lai_x, qsintmax, deadleaf_cover, assim_param_x)
      ENDIF

      !
      ! 1.10 reset flag
      !

      l_first_stomate = .FALSE.

      !
      ! 1.11 return
      !

      RETURN

    ENDIF  ! first call

    !
    ! 2 prepares restart file for the next simulation
    !
    IF (ldrestart_write) THEN

      IF (long_print) WRITE (numout,*) ' we have to complete restart file with STOMATE variables '

      CALL writerestart (kjpindex, index, &
                         day_counter, dt_days, date, &
                         ind, adapted, regenerate, &
                         humrel_daily, litterhum_daily, &
                         t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
                         soilhum_daily, precip_daily, &
                         gpp_daily, npp_daily, turnover_daily, &
                         humrel_month, humrel_week, t2m_longterm, tlong_ref, t2m_month, t2m_week, &
                         tsoil_month, soilhum_month, fireindex, firelitter, &
                         maxhumrel_lastyear, maxhumrel_thisyear, &
                         minhumrel_lastyear, minhumrel_thisyear, &
                         maxgppweek_lastyear, maxgppweek_thisyear, &
                         gdd0_lastyear, gdd0_thisyear, &
                         precip_lastyear, precip_thisyear, &
                         gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
                         PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
                         maxfpc_lastyear, maxfpc_thisyear, &
                         turnover_longterm, gpp_week, biomass, &
                         fvm, fv, leaf_age, leaf_frac, &
                         senescence, when_growthinit, age, &
                         resp_hetero, resp_maint, resp_growth, co2_fire, co2_to_bm_dgvm, &
                         veget_lastlight, everywhere, need_adjacent, RIP_time, time_lowgpp, &
                         time_hum_min, hum_min_dormance, &
                         litterpart, litter, dead_leaves, &
                         carbon, black_carbon, lignin_struc,turnover_time)

      IF ( ldforcing_write) THEN
        !
        CALL forcing_write(forcing_id,1,iisf)
        !
        ier = NF_CLOSE( forcing_id )
        forcing_id=-1
        !
      ENDIF

      IF ( ldcarbon_write ) THEN
        !
        WRITE(numout,*) 'stomate: writing the forcing file for carbon spinup'
        !
        DO iatt = 1, nparan
          IF ( nforce(iatt) .GT. 0 ) THEN
            soilcarbon_input(:,:,:,iatt) = soilcarbon_input(:,:,:,iatt) / FLOAT(nforce(iatt))
            control_moist(:,:,iatt) = control_moist(:,:,iatt) / FLOAT(nforce(iatt))
            control_temp(:,:,iatt) = control_temp(:,:,iatt) / FLOAT(nforce(iatt))
          ELSE
            WRITE(numout,*) 'We have no soil carbon forcing data for this time step:',iatt
            WRITE(numout,*) ' -> we set them to zero'
            soilcarbon_input(:,:,:,iatt) = zero
            control_moist(:,:,iatt) = zero
            control_temp(:,:,iatt) = zero
          ENDIF
        ENDDO
        !
        !Config  Key  = STOMATE_CFORCING_NAME
        !Config  Desc = Name of STOMATE's carbon forcing file
        !Config  Def  = stomate_Cforcing.nc
        !Config  Help = Name that will be given to STOMATE's carbon 
        !Config         offline forcing file

        Cforcing_name = stomate_Cforcing_name
        CALL getin('STOMATE_CFORCING_NAME', Cforcing_name)

        CALL SYSTEM( 'rm -f '//TRIM(Cforcing_name) )

        ier = NF_CREATE( TRIM(Cforcing_name), NF_WRITE, Cforcing_id )

        ier = NF_PUT_ATT_STND( Cforcing_id, NF_GLOBAL, 'kjpindex', REAL(kjpindex,stnd) )
        ier = NF_PUT_ATT_STND( Cforcing_id, NF_GLOBAL, 'nparan', REAL(nparan,stnd) )
        numdim=1
        ier = NF_DEF_DIM( Cforcing_id, 'points', kjpindex, numdim)
        numdim=2
        ier = NF_DEF_DIM( Cforcing_id, 'carbtype', ncarb, numdim )
        numdim=3
        ier = NF_DEF_DIM( Cforcing_id, 'vegtype', nvegtypes, numdim )
        numdim=4
        ier = NF_DEF_DIM( Cforcing_id, 'level', nlevs, numdim)
        numdim=5
        ier = NF_DEF_DIM( Cforcing_id, 'time_step', nparan, numdim )

        ndim = 1 ; idim(1) = 1 ; varname = 'points'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 2 ; varname = 'carbtype'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 3 ; varname = 'vegtype'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 4 ; varname = 'level'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 5 ; varname = 'time_step'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 1 ; varname = 'index'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 1 ; varname = 'clay'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 1 ; idim(1) = 1 ; varname = 'space_nat'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 4 ; idim(1) = 1 ; idim(2) = 2 ; idim(3) = 3 ; idim(4) = 5 ;
        varname = 'soilcarbon_input'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 3 ; idim(1) = 1 ; idim(2) = 4 ; idim(3) = 5 ; varname = 'control_moist'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ndim = 3 ; idim(1) = 1 ; idim(2) = 4 ; idim(3) = 5 ; varname = 'control_temp'
        ier = NF_DEF_VAR_STND( Cforcing_id, TRIM(varname), ndim, idim, vid )

        ier = NF_ENDDEF( Cforcing_id )

        ier = NF_PUT_VAR_STND( Cforcing_id, 'points', (/ (REAL(i,stnd),i=1,kjpindex) /) )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'carbtype', (/ (REAL(i,stnd),i=1,ncarb) /) )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'vegtype', (/ (REAL(i,stnd),i=1,nvegtypes) /) )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'level', (/ (REAL(i,stnd),i=1,nlevs) /) )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'time_step', (/ (REAL(i,stnd),i=1,nparan) /) )

        ier = NF_PUT_VAR_STND( Cforcing_id, 'index', REAL(index,stnd) )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'clay', clay )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'space_nat', space_nat )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'soilcarbon_input', soilcarbon_input )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'control_moist', control_moist )
        ier = NF_PUT_VAR_STND( Cforcing_id, 'control_temp', control_temp )

        ier = NF_CLOSE( Cforcing_id )
        forcing_id=-1
      ENDIF

      RETURN

    END IF  ! write restart-files

    !
    ! 3 Here's where the serious things begin -
    !   Check whether stomate and dgvm have to be called
    !

    !
    ! 3.1 update day counter
    !

    day_counter = day_counter + dtradia

    IF ( NINT(day_counter) .GE. NINT(dt_slow) ) THEN

      !
      ! 3.2 we have to call STOMATE
      !     (or daily update of vegetation characteristics)?
      !

      ! reset counter
      day_counter = zero

      do_slow = .TRUE.

      !
      ! 3.3 is one year over?
      !     EndOfYear must be true once per year during a call of stomate_season.
      !

      ! increase date
      dt_days = dt_slow / one_day
      date = date + dt_days

      IF ( ( ( date - NINT(date/one_year)*one_year ) .LT.  .0 ) .AND. &
           ( ( date - NINT(date/one_year)*one_year ) .GE. -dt_days )        ) THEN
        EndOfYear = .TRUE.
      ELSE
        EndOfYear = .FALSE.
      ENDIF

    ELSE

      do_slow = .FALSE.

      EndOfYear = .FALSE.

    ENDIF

    IF ( ( ldcarbon_write ) .AND. &
         ( ( date - NINT(date/one_year)*one_year ) .EQ. dt_days )        ) THEN
        control_moist(:,:,:) = zero
        control_temp(:,:,:) = zero
        soilcarbon_input(:,:,:,:) = zero
        nforce=0
     
     ENDIF

    !
    ! 4 Special treatment for some input arrays.
    !

    !
    ! 4.1 Sum of liquid and solid precipitation
    !

    precip = ( precip_rain + precip_snow ) * one_day/dtradia

    !
    ! 4.2 Transform from dimension nvm to dimension npft.
    !     In SECHIBA, some variables are defined for all PFTs AND on bare ground -> nvm
    !     In STOMATE/LPJ, we do not treat bare ground. The same variable need not be
    !       defined on bare ground in STOMATE/LPJ  -> npft
    !     Moreover, several Stomate-PFTs can be aggregated to one single PFT in Sechiba.
    !     This is not used for the moment, but it might be used once an age structure
    !       is introduced in Stomate. In that case, the different age groups of a PFT may
    !       be treated separately in Stomate, but as a single PFT in Sechiba.
    !     The corresponding variables exchanged with the rest of SECHIBA are tagged _x.
    !     XXX_x(:,ibare_sechiba) is the bare soil part.
    !     ipft_sechiba(j) is the Sechiba PFT index corresponding
    !       to the Stomate PFT index j.
    !

    ! 4.2.1 calculate STOMATE's vegetation fractions from veget_x, veget_max_x, fv, and fvm

    CALL stomate_vegconvert (kjpindex,'in',fraction_nobio, &
                             veget_max_x,veget_x,veget_max,veget,fvm,fv)

    DO j = 1, npft

      ! 4.2.2 GPP
      ! gpp in gC/m**2 of total ground/day

      WHERE ( veget_max_x(:,ipft_sechiba(j)) .GT. 0.0 )
        gpp(:,j) = gpp_x(:,ipft_sechiba(j)) * one_day/dtradia * &
                   ( veget_max(:,j) / veget_max_x(:,ipft_sechiba(j)) )
      ELSEWHERE
        gpp(:,j) = 0.0
      ENDWHERE

      ! 4.2.3 moisture availability

      humrel(:,j) = humrel_x(:,ipft_sechiba(j))

    ENDDO

     space_nat(:) = 1.0

      DO j = 1, npft
        IF ( .NOT. natural(j) ) THEN
          space_nat(:) = space_nat(:) - veget_max(:,j)
        ENDIF
      ENDDO
   !
    ! 5 "daily" variables
    !   Note: If dt_days .NE. 1, then xx_daily are not daily variables, but
    !         that is not really a problem.
    !

    !
    ! 5.1 accumulate instantaneous variables and eventually calculate daily mean value
    !

    CALL stomate_accu (kjpindex, npft, dt_slow, dtradia, do_slow, humrel, humrel_daily)
    CALL stomate_accu (kjpindex, 1, dt_slow, dtradia, do_slow, litterhumdiag, litterhum_daily)
    CALL stomate_accu (kjpindex, 1, dt_slow, dtradia, do_slow, t2m, t2m_daily)
    CALL stomate_accu (kjpindex, 1, dt_slow, dtradia, do_slow, temp_sol, tsurf_daily)
    CALL stomate_accu (kjpindex, nbdl, dt_slow, dtradia, do_slow, stempdiag, tsoil_daily)
    CALL stomate_accu (kjpindex, nbdl, dt_slow, dtradia, do_slow, shumdiag, soilhum_daily)
    CALL stomate_accu (kjpindex, 1, dt_slow, dtradia, do_slow, precip, precip_daily)
    CALL stomate_accu (kjpindex, npft, dt_slow, dtradia, do_slow, gpp, gpp_daily)

    !
    ! 5.2 daily minimum temperature
    !

    t2m_min_daily(:) = MIN( t2m_min(:), t2m_min_daily(:) )

    !
    ! 6 Daily processes
    !

    IF ( do_slow ) THEN

       ! 6.0 update lai
       if (control%ok_pheno) THEN ! lai from stomate
          DO j = 1, npft
             WHERE ( veget_max(:,j) .GT. 0.0 )
                lai(:,j) = biomass(:,j,ileaf) / veget_max(:,j) * sla(j)
             ELSEWHERE
                lai(:,j) = 0.0
             ENDWHERE
          ENDDO
       ELSE
          CALL  setlai(kjpindex,lai) ! lai prescribed
       ENDIF



      !
      ! 6.1 total natural space
      !

  
      !
      ! 6.2 Calculate longer-term "meteorological" and biological parameters
      !

      CALL season (kjpindex, dt_days, EndOfYear, space_nat, &
                   veget, veget_max, &
                   humrel_daily, t2m_daily, tsoil_daily, soilhum_daily, &
                   precip_daily, npp_daily, biomass, turnover_daily, gpp_daily, when_growthinit, &
                   maxhumrel_lastyear, maxhumrel_thisyear, &
                   minhumrel_lastyear, minhumrel_thisyear, &
                   maxgppweek_lastyear, maxgppweek_thisyear, &
                   gdd0_lastyear, gdd0_thisyear, &
                   precip_lastyear, precip_thisyear, &
                   lm_lastyearmax, lm_thisyearmax, &
                   maxfpc_lastyear, maxfpc_thisyear, &
                   humrel_month, humrel_week, t2m_longterm, tlong_ref, t2m_month, t2m_week, &
                   tsoil_month, soilhum_month, &
                   npp_longterm, turnover_longterm, gpp_week, &
                   gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, time_lowgpp, &
                   time_hum_min, hum_min_dormance, herbivores)

      !
      ! 6.3 transform GPP from gC/(m**2 of total ground)/day to
      !     gC/(m**2 of nat/agri ground)/day
      !

      CALL natagritot (kjpindex, ito_natagri, space_nat, gpp_daily)

      !
      ! 6.4 STOMATE: allocation, phenology, etc.
      !

      IF ( control%ok_stomate ) THEN

        ! 6.4.1 transform spatial fractions from fraction of total space to fraction of
        !       natural/agricultural space

        CALL natagritot (kjpindex, ito_natagri, space_nat, veget)

        CALL natagritot (kjpindex, ito_natagri, space_nat, veget_max)

        ! 6.4.2 call stomate

        CALL StomateLpj (kjpindex, dt_days, &
                         neighbours, resolution, space_nat, &
                         clay, herbivores, &
                         tsurf_daily, tsoil_daily, t2m_daily, t2m_min_daily, &
                         litterhum_daily, soilhum_daily, &
                         maxhumrel_lastyear, minhumrel_lastyear, &
                         gdd0_lastyear, precip_lastyear, &
                         humrel_month, humrel_week, tlong_ref, t2m_month, t2m_week, &
                         tsoil_month, soilhum_month, &
                         gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
                         turnover_longterm, gpp_daily, time_lowgpp, &
                         time_hum_min, maxfpc_lastyear, &
                         PFTpresent, age, fireindex, firelitter, &
                         leaf_age, leaf_frac, biomass, ind, adapted, regenerate, &
                         senescence, when_growthinit, &
                         litterpart, litter, dead_leaves, carbon, black_carbon, lignin_struc, &
                         veget_max, veget, npp_longterm, lm_lastyearmax, veget_lastlight, &
                         everywhere, need_adjacent, RIP_time, &
                         lai, npp_daily, turnover_daily, turnover_time,&
                         control_moist_inst, control_temp_inst, soilcarbon_input_inst, &
                         co2_to_bm_dgvm, co2_fire, resp_hetero, resp_maint, resp_growth, &
                         height, deadleaf_cover, vcmax, vjmax, t_photo_min, t_photo_opt, t_photo_max)

        ! 6.4.3 transform spatial fractions from fraction of natural/agricultural
        !       space to fraction of total space

        CALL natagritot (kjpindex, ito_total, space_nat, veget)

        CALL natagritot (kjpindex, ito_total, space_nat, veget_max)

        !
        ! 6.5 output: transform from dimension npft to dimension nvm
        !       Several Stomate-PFTs may correspond to a single Sechiba-PFT (see 4.2).
        !       We sum up the vegetation cover over these Stomate-PFTs.
        !       Mean LAI, height, and Vmax is calculated by ponderating with (maximum)
        !       vegetation cover.

        ! 6.5.1 calculate veget_x, veget_max_x, fv and fvm from veget and veget_max
        CALL stomate_vegconvert (kjpindex,'out',fraction_nobio, &
                                 veget_max_x,veget_x,veget_max,veget,fvm,fv)

        ! 6.5.2 lai and height

        CALL stomate_var_xout (kjpindex,lai,veget_max,zero,lai_x)
        CALL stomate_var_xout (kjpindex,height,veget_max,zero,height_x)

        ! 6.5.3 photosynthesis parameters

        CALL stomate_var_xout (kjpindex,vcmax,veget,zero,assim_param_x(:,:,ivcmax))
        CALL stomate_var_xout (kjpindex,vjmax,veget,zero,assim_param_x(:,:,ivjmax))

        CALL stomate_var_xout (kjpindex,t_photo_min,veget,zero,assim_param_x(:,:,itmin))
        CALL stomate_var_xout (kjpindex,t_photo_opt,veget,zero,assim_param_x(:,:,itopt))
        CALL stomate_var_xout (kjpindex,t_photo_max,veget,zero,assim_param_x(:,:,itmax))

        !
        ! 6.6 update forcing variables for soil carbon
        !

        IF ( ldcarbon_write ) THEN
  
          ! deterime the carbon soil time step we are falling into
  
          ! where are we within the current year?
          sf_time = date - NINT(date/one_year)*one_year
          DO WHILE ( sf_time .GT. one_year ) 
            sf_time = sf_time - one_year
          ENDDO
          DO WHILE ( sf_time .LT. 0. ) 
            sf_time = sf_time + one_year
          ENDDO
          IF (iatt .GT. nparan) iatt=1
          IF ( ( times(iatt-1) .GT. sf_time ) .OR. ( times(iatt) .LE. sf_time ) ) THEN
            iatt = nparan
            ! look for corresponding time step
            DO i = nparan, 1, -1
              IF ( times(i) .GT. sf_time ) iatt = i
            ENDDO
          ENDIF ! otherwise, iatt was already OK!
  
          ! we know now what soil forcing time step we are talking about.
          ! Increase counter for this soil carbon time step and update "mean" 
          ! forcing variables.
  
          nforce(iatt) = nforce(iatt) + 1
  
          soilcarbon_input(:,:,:,iatt) = &
            soilcarbon_input(:,:,:,iatt) + soilcarbon_input_inst(:,:,:)
          control_moist(:,:,iatt) = control_moist(:,:,iatt) + control_moist_inst(:,:)
          control_temp(:,:,iatt) = control_temp(:,:,iatt) + control_temp_inst(:,:)

        ENDIF

        !
        ! 6.7 updates qsintmax
        !

        qsintmax(:,:) = qsintcst * veget_x(:,:) * lai_x(:,:)

      ENDIF

      !
      ! 6.7 write forcing file?
      !     ldforcing_write should only be .TRUE. if STOMATE is run in coupled mode.
      !     In stand-alone mode, the forcing file is read!
      !

      IF ( ldforcing_write ) THEN

        CALL natagritot (kjpindex, ito_total, space_nat, gpp_daily)

        gpp_daily_x(:,:) = 0.0

        DO j = 1, npft

          ! don't worry about weightings. humrel is not modified in STOMATE.
          humrel_daily_x(:,ipft_sechiba(j)) = humrel_daily(:,j)

          gpp_daily_x(:,ipft_sechiba(j)) = gpp_daily_x(:,ipft_sechiba(j)) + &
                                           gpp_daily(:,j) * dt_slow/one_day

        ENDDO

        ! bare soil moisture availability has not been treated in STOMATE (doesn't matter)
        humrel_daily_x(:,ibare_sechiba) = humrel_x(:,ibare_sechiba)

        ! next forcing step in memory
        iisf = iisf + 1

        ! how many times have we treated this forcing state
        xn = FLOAT(nf_cumul(isf(iisf)))

        ! cumulate. be careful: precipitation is multiplied by dt_slow/one_day
        IF (cumul_forcing) THEN
           clay_fm(:,iisf) = ( xn*clay_fm(:,iisf) + clay(:) ) / (xn+1.)
           humrel_daily_x_fm(:,:,iisf) = ( xn*humrel_daily_x_fm(:,:,iisf) + humrel_daily_x(:,:) ) / (xn+1.)
           litterhum_daily_fm(:,iisf) = ( xn*litterhum_daily_fm(:,iisf) + litterhum_daily(:) ) / (xn+1.)
           t2m_daily_fm(:,iisf) = ( xn*t2m_daily_fm(:,iisf) + t2m_daily(:) ) / (xn+1.)
           t2m_min_daily_fm(:,iisf) = ( xn*t2m_min_daily_fm(:,iisf) + t2m_min_daily(:) ) / (xn+1.)
           tsurf_daily_fm(:,iisf) = ( xn*tsurf_daily_fm(:,iisf) + tsurf_daily(:) ) / (xn+1.)
           tsoil_daily_fm(:,:,iisf) = ( xn*tsoil_daily_fm(:,:,iisf) + tsoil_daily(:,:) ) / (xn+1.)
           soilhum_daily_fm(:,:,iisf) = ( xn*soilhum_daily_fm(:,:,iisf) + soilhum_daily(:,:) ) / (xn+1.)
           precip_fm(:,iisf) = ( xn*precip_fm(:,iisf) + precip_daily(:) * dt_slow/one_day ) / (xn+1.)
           gpp_daily_x_fm(:,:,iisf) = ( xn*gpp_daily_x_fm(:,:,iisf) + gpp_daily_x(:,:) ) / (xn+1.)
           veget_x_fm(:,:,iisf) = ( xn*veget_x_fm(:,:,iisf) + veget_x(:,:) ) / (xn+1.)
           veget_max_x_fm(:,:,iisf) = ( xn*veget_max_x_fm(:,:,iisf) + veget_max_x(:,:) ) / (xn+1.)
           lai_x_fm(:,:,iisf) = ( xn*lai_x_fm(:,:,iisf) + lai_x(:,:) ) / (xn+1.)
        ELSE
           clay_fm(:,iisf) =clay(:)
           humrel_daily_x_fm(:,:,iisf) =  humrel_daily_x(:,:)
           litterhum_daily_fm(:,iisf) =  + litterhum_daily(:)
           t2m_daily_fm(:,iisf) = t2m_daily(:) 
           t2m_min_daily_fm(:,iisf) =t2m_min_daily(:) 
           tsurf_daily_fm(:,iisf) = tsurf_daily(:) 
           tsoil_daily_fm(:,:,iisf) =tsoil_daily(:,:)
           soilhum_daily_fm(:,:,iisf) =soilhum_daily(:,:)
           precip_fm(:,iisf) = precip_daily(:) 
           gpp_daily_x_fm(:,:,iisf) =gpp_daily_x(:,:)
           veget_x_fm(:,:,iisf) = veget_x(:,:)
           veget_max_x_fm(:,:,iisf) =veget_max_x(:,:) 
           lai_x_fm(:,:,iisf) =lai_x(:,:)
        END IF
        nf_cumul(isf(iisf)) = nf_cumul(isf(iisf)) + 1

        ! do we have to write the forcing states?

        IF ( iisf .EQ. nsfm ) THEN

          ! write these forcing states
          CALL forcing_write(forcing_id,1,nsfm)

          ! determine which forcing states must be read

          isf(1) = isf(nsfm) + 1
          IF ( isf(1) .GT. nsft ) isf(1) = 1
          DO iisf = 2, nsfm
            isf(iisf) = isf(iisf-1) + 1
            IF ( isf(iisf) .GT. nsft ) isf(iisf) = 1
          ENDDO

          ! read them
          CALL forcing_read(forcing_id,nsfm)

          iisf = 0

        ENDIF

      ENDIF
      ! 6.8 compute daily co2_flux
     resp_auto_tot(:) = 0.0

      DO j = 1, npft
        resp_auto_tot(:) = resp_auto_tot(:) + resp_maint(:,j) + resp_growth(:,j)
      ENDDO

      ! total photosynthesis (in gC/m**2/day)

      gpp_tot(:) = 0.0

      DO j = 1, npft
        gpp_tot(:) = gpp_tot(:) + gpp_daily(:,j)
      ENDDO

      ! CO2 flux in gC/m**2/sec (positive towards the atmosphere) is sum of:
      ! 1/ heterotrophic respiration from natural and agricultural ground
      ! 2/ maintenance respiration from the plants
      ! 3/ growth respiration from the plants
      ! 4/ co2 created by fire
      ! 5/ - co2 taken up in the DGVM to establish saplings.
      ! 6/ - co2 taken up by photosyntyhesis

      co2_flux_daily(:) = ( resp_hetero(:,inat) + &
                      resp_hetero(:,iagri) + &
                      resp_auto_tot(:) + &
                      co2_fire(:) - co2_to_bm_dgvm(:) - gpp_tot(:) )      
         CALL histwrite (hist_id_stomate, 'CO2FLUX', itime, &
              co2_flux_daily, kjpindex, hori_index)
 
      !
      ! 6.9 reset daily variables
      !

      humrel_daily(:,:) = 0.0
      litterhum_daily(:) = 0.0
      t2m_daily(:) = 0.0
      t2m_min_daily(:) = large_value
      tsurf_daily(:) = 0.0
      tsoil_daily(:,:) = 0.0
      soilhum_daily(:,:) = 0.0
      precip_daily(:) = 0.0
      gpp_daily(:,:) = 0.0

      IF ( bavard .GE. 3 ) WRITE(numout,*) 'stomate_main: daily processes done'

    ENDIF  ! daily processes?

    !
    ! 7 Outputs from Stomate
    !   co2_flux is assigned a value only if STOMATE is activated.
    !   Otherwise, the calling hydrological module must do this itself.
    !

    IF ( control%ok_stomate ) THEN

      ! total autotrophic respiration

      resp_auto_tot(:) = 0.0

      DO j = 1, npft
        resp_auto_tot(:) = resp_auto_tot(:) + resp_maint(:,j) + resp_growth(:,j)
      ENDDO

      ! total photosynthesis (in gC/m**2/day)

      gpp_tot(:) = 0.0

      DO jv = 1, nvm
        gpp_tot(:) = gpp_tot(:) + gpp_x(:,jv) * one_day / dtradia
      ENDDO

      ! CO2 flux in gC/m**2/sec (positive towards the atmosphere) is sum of:
      ! 1/ heterotrophic respiration from natural and agricultural ground
      ! 2/ maintenance respiration from the plants
      ! 3/ growth respiration from the plants
      ! 4/ co2 created by fire
      ! 5/ - co2 taken up in the DGVM to establish saplings.
      ! 6/ - co2 taken up by photosyntyhesis

      co2_flux(:) = ( resp_hetero(:,inat) + &
                      resp_hetero(:,iagri) + &
                      resp_auto_tot(:) + &
                      co2_fire(:) - co2_to_bm_dgvm(:) - gpp_tot(:) ) / one_day
      
   ENDIF

    !
    ! 8 message
    !

    IF ( ( bavard .GE. 2 ) .AND. EndOfYear .AND. do_slow ) THEN
      WRITE(numout,*) 'stomate: EndOfYear'
    ENDIF

    IF ( bavard .GE. 4 ) WRITE(numout,*) 'Leaving stomate_main'

    IF (long_print) WRITE (numout,*) ' stomate_main done '

  END SUBROUTINE stomate_main

  SUBROUTINE stomate_init (kjpij, kjpindex, index, ldforcing_write, lalo, &
                            rest_id_stom, hist_id_stom)

    ! interface description
    ! input scalar
    ! Total size of the un-compressed grid
    INTEGER(long), INTENT(in)                               :: kjpij
    ! Domain size
    INTEGER(long), INTENT (in)                              :: kjpindex
    ! Logical for _forcing_ file to write
    LOGICAL, INTENT(in)                                     :: ldforcing_write
    ! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (in)          :: lalo
    ! STOMATE's _Restart_ file file identifier
    INTEGER(long),INTENT (in)                               :: rest_id_stom
    ! STOMATE's _history_ file file identifier
    INTEGER(long),INTENT (in)                               :: hist_id_stom
    ! input fields
    ! Indeces of the points on the map
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)         :: index

    ! local declaration
    REAL(stnd)                                              :: tmp_day(1)
    ! soil depth taken for canopy
    REAL(stnd)                                              :: zcanop
    ! soil depths at diagnostic levels
    REAL(stnd), DIMENSION(nbdl)                             :: zsoil
    ! Index 
    INTEGER(short)                                          :: l
    INTEGER(short), DIMENSION(1)                            :: vtmp
    ! To store variables names for I/O
    CHARACTER*80                                            :: var_name
    ! allocation error
    LOGICAL                                                 :: l_error
    ! Global world fraction of vegetation type map
    REAL(stnd), DIMENSION(360,180,nvm)                      :: veget_ori_on_disk
    INTEGER(short)                                          :: ier
    ! indices
    INTEGER(long)                                           :: ji,j,ipd

    !
    ! 1 online diagnostics (by default, "bavard" is set to 1 in stomate_constants)
    !

    !
    !Config  Key  = BAVARD
    !Config  Desc = level of online diagnostics in STOMATE (0-4)
    !Config  Def  = 1
    !Config  Help = With this variable, you can determine
    !Config         how much online information STOMATE
    !Config         gives during the run. 0 means
    !Config         virtually no info.
    !
    bavard = 1
    CALL getin('BAVARD', bavard)

    IF ( kjpindex .GT. 0 ) THEN
      !
      !Config  Key  = STOMATE_DIAGPT
      !Config  Desc = Index of grid point for online diagnostics
      !Config  Def  = 1
      !Config  Help = This is the index of the grid point which
      !               will be used for online diagnostics.
      ipd = 1
      CALL getin('STOMATE_DIAGPT',ipd)
      ipd = MIN( ipd, kjpindex )
      WRITE(numout,*) 'Stomate: '
      WRITE(numout,*) '  Index of grid point for online diagnostics: ',ipd
      WRITE(numout,*) '  Lon, lat:',lalo(ipd,2),lalo(ipd,1)
      WRITE(numout,*) '  Index of this point on GCM grid: ',index(ipd)
      !
    ENDIF

    !
    ! 2 check consistency of flags
    !

    IF ( ( .NOT. control%ok_stomate ) .AND. control%ok_dgvm ) THEN
      WRITE(numout,*) 'Cannot do dynamical vegetation without STOMATE.'
      WRITE(numout,*) 'We stop.'
      STOP
    ENDIF

    IF ( ( .NOT. control%ok_co2 ) .AND. control%ok_stomate ) THEN
      WRITE(numout,*) 'Cannot call STOMATE without GPP.'
      WRITE(numout,*) 'We stop.'
      STOP
    ENDIF

    IF ( ( .NOT. control%ok_co2 ) .AND. ldforcing_write ) THEN
      WRITE(numout,*) 'Cannot write forcing file if photosynthesis is not activated'
      WRITE(numout,*) 'We stop.'
      STOP
    ENDIF

    !
    ! 3 messages
    !

    WRITE(numout,*) 'stomate: first call'
    WRITE(numout,*) '  Photosynthesis: ', control%ok_co2
    WRITE(numout,*) '  STOMATE: ', control%ok_stomate
    WRITE(numout,*) '  LPJ: ', control%ok_dgvm

    !
    ! 4 allocation of STOMATE's variables
    !

    l_error = .FALSE.
    ALLOCATE (veget_max(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (space_nat(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (ind(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (adapted(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (regenerate(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (humrel_daily(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (litterhum_daily(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_daily(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_min_daily(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (tsurf_daily(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (tsoil_daily(kjpindex,nbdl), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (soilhum_daily(kjpindex,nbdl), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (precip_daily(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gpp_daily(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (npp_daily(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (turnover_daily(kjpindex,npft,nparts), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (humrel_month(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (humrel_week(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_longterm(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (tlong_ref(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_month(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_week(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (tsoil_month(kjpindex,nbdl), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (soilhum_month(kjpindex,nbdl), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (fireindex(kjpindex,nvegtypes), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (firelitter(kjpindex,nvegtypes), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (maxhumrel_lastyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (maxhumrel_thisyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (minhumrel_lastyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (minhumrel_thisyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (maxgppweek_lastyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (maxgppweek_thisyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gdd0_lastyear(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gdd0_thisyear(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (precip_lastyear(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (precip_thisyear(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gdd_m5_dormance(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gdd_midwinter(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (ncd_dormance(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (ngd_minus5(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (PFTpresent(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (npp_longterm(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (lm_lastyearmax(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (lm_thisyearmax(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (maxfpc_lastyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (maxfpc_thisyear(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (turnover_longterm(kjpindex,npft,nparts), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gpp_week(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (biomass(kjpindex,npft,nparts), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (senescence(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (when_growthinit(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (age(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (resp_hetero(kjpindex,nvegtypes), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (resp_maint(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (resp_growth(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (co2_fire(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (co2_to_bm_dgvm(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (veget_lastlight(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (everywhere(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (need_adjacent(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (leaf_age(kjpindex,npft,nleafages), stat=ier); l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (leaf_frac(kjpindex,npft,nleafages), stat=ier); l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (RIP_time(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (time_lowgpp(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (time_hum_min(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (hum_min_dormance(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (fvm(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (fv(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (litterpart(kjpindex,npft,nlitt), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (litter(kjpindex,nlitt,nvegtypes,nlevs), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (dead_leaves(kjpindex,npft,nlitt), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (carbon(kjpindex,ncarb,nvegtypes), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (black_carbon(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (lignin_struc(kjpindex,nvegtypes,nlevs), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (turnover_time(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (co2_flux_daily(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)

    ALLOCATE (herbivores(kjpindex,npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)

    ALLOCATE (hori_index(kjpindex), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (horipft_index(kjpindex*npft), stat=ier) ; l_error = l_error .OR. (ier.NE.0)

    IF ( l_error ) THEN
      STOP 'stomate_init: error in memory allocation'
    ENDIF

    !
    ! 5 file definitions: stored in common variables
    !

    hist_id_stomate = hist_id_stom

    rest_id_stomate = rest_id_stom

    hori_index(:) = index(:)

    ! Get the indexing table for the vegetation fields. In STOMATE we work on
    ! reduced grids but to store in the full 3D filed vegetation variable
    ! we need another index table : indexpft

    DO j = 1, npft
      DO ji = 1, kjpindex
        horipft_index((j-1)*kjpindex + ji) = index(ji) + (j-1)*kjpij
      ENDDO
    ENDDO

    !
    ! 6 some flags
    !

    !Config  Key  = AGRICULTURE
    !Config  Desc = agriculture allowed?
    !Config  Def  = y
    !Config  Help = With this variable, you can determine
    !Config         whether agriculture is allowed
    !
    agriculture = .TRUE.
    CALL getin('AGRICULTURE', agriculture)

    WRITE(numout,*) 'agriculture: ',agriculture

    !Config  Key  = HERBIVORES
    !Config  Desc = herbivores allowed?
    !Config  Def  = n
    !Config  Help = With this variable, you can determine
    !Config         if herbivores are activated
    !
    ok_herbivores = .FALSE.
    CALL getin('HERBIVORES', ok_herbivores)

    WRITE(numout,*) 'herbivores activated: ',ok_herbivores

    !Config  Key  = TREAT_EXPANSION
    !Config  Desc = treat expansion of PFTs across a grid cell?
    !Config  Def  = n
    !Config  Help = With this variable, you can determine
    !Config         whether we treat expansion of PFTs across a
    !Config         grid cell.
    !
    treat_expansion = .FALSE.
    CALL getin('TREAT_EXPANSION', treat_expansion)

    WRITE(numout,*) 'expansion across a grid cell is treated: ',treat_expansion

  END SUBROUTINE stomate_init
  SUBROUTINE stomate_clear
    
    !1   . Deallocate all dynamics variables  
    IF ( ALLOCATED (veget_max)) DEALLOCATE (veget_max)
    IF ( ALLOCATED (space_nat)) DEALLOCATE (space_nat)
    IF ( ALLOCATED (ind)) DEALLOCATE (ind)
    IF ( ALLOCATED (adapted)) DEALLOCATE (adapted)
    IF ( ALLOCATED (regenerate)) DEALLOCATE (regenerate)
    IF ( ALLOCATED (humrel_daily))  DEALLOCATE (humrel_daily) 
    IF ( ALLOCATED (litterhum_daily)) DEALLOCATE (litterhum_daily)
    IF ( ALLOCATED (t2m_daily))  DEALLOCATE (t2m_daily)
    IF ( ALLOCATED (t2m_min_daily))  DEALLOCATE (t2m_min_daily)
    IF ( ALLOCATED (tsurf_daily))  DEALLOCATE (tsurf_daily)
    IF ( ALLOCATED (tsoil_daily)) DEALLOCATE (tsoil_daily)
    IF ( ALLOCATED (soilhum_daily)) DEALLOCATE (soilhum_daily)
    IF ( ALLOCATED (precip_daily)) DEALLOCATE (precip_daily)
    IF ( ALLOCATED (gpp_daily)) DEALLOCATE (gpp_daily)
    IF ( ALLOCATED (npp_daily)) DEALLOCATE (npp_daily)
    IF ( ALLOCATED (turnover_daily)) DEALLOCATE (turnover_daily)
    IF ( ALLOCATED (humrel_month)) DEALLOCATE (humrel_month)
    IF ( ALLOCATED (humrel_week)) DEALLOCATE (humrel_week)
    IF ( ALLOCATED (t2m_longterm)) DEALLOCATE (t2m_longterm)
    IF ( ALLOCATED (tlong_ref)) DEALLOCATE (tlong_ref)
    IF ( ALLOCATED (t2m_month)) DEALLOCATE (t2m_month)
    IF ( ALLOCATED (t2m_week)) DEALLOCATE (t2m_week)
    IF ( ALLOCATED (tsoil_month)) DEALLOCATE (tsoil_month)
    IF ( ALLOCATED (soilhum_month)) DEALLOCATE (soilhum_month)
    IF ( ALLOCATED (fireindex)) DEALLOCATE (fireindex)
    IF ( ALLOCATED (firelitter)) DEALLOCATE (firelitter)
    IF ( ALLOCATED (maxhumrel_lastyear)) DEALLOCATE (maxhumrel_lastyear)
    IF ( ALLOCATED (maxhumrel_thisyear)) DEALLOCATE (maxhumrel_thisyear)
    IF ( ALLOCATED (minhumrel_lastyear)) DEALLOCATE (minhumrel_lastyear)
    IF ( ALLOCATED (minhumrel_thisyear)) DEALLOCATE (minhumrel_thisyear)
    IF ( ALLOCATED (maxgppweek_lastyear)) DEALLOCATE (maxgppweek_lastyear)
    IF ( ALLOCATED (maxgppweek_thisyear)) DEALLOCATE (maxgppweek_thisyear)
    IF ( ALLOCATED (gdd0_lastyear)) DEALLOCATE (gdd0_lastyear)
    IF ( ALLOCATED (gdd0_thisyear)) DEALLOCATE (gdd0_thisyear)
    IF ( ALLOCATED (precip_lastyear)) DEALLOCATE (precip_lastyear)
    IF ( ALLOCATED (precip_thisyear)) DEALLOCATE (precip_thisyear)
    IF ( ALLOCATED (gdd_m5_dormance)) DEALLOCATE (gdd_m5_dormance)
    IF ( ALLOCATED (gdd_midwinter)) DEALLOCATE (gdd_midwinter)
    IF ( ALLOCATED (ncd_dormance)) DEALLOCATE (ncd_dormance)
    IF ( ALLOCATED (ngd_minus5))  DEALLOCATE (ngd_minus5)
    IF ( ALLOCATED (PFTpresent)) DEALLOCATE (PFTpresent)
    IF ( ALLOCATED (npp_longterm)) DEALLOCATE (npp_longterm)
    IF ( ALLOCATED (lm_lastyearmax)) DEALLOCATE (lm_lastyearmax)
    IF ( ALLOCATED (lm_thisyearmax)) DEALLOCATE (lm_thisyearmax)
    IF ( ALLOCATED (maxfpc_lastyear)) DEALLOCATE (maxfpc_lastyear)
    IF ( ALLOCATED (maxfpc_thisyear)) DEALLOCATE (maxfpc_thisyear)
    IF ( ALLOCATED (turnover_longterm)) DEALLOCATE (turnover_longterm)
    IF ( ALLOCATED (gpp_week)) DEALLOCATE (gpp_week)
    IF ( ALLOCATED (biomass)) DEALLOCATE (biomass)
    IF ( ALLOCATED (senescence)) DEALLOCATE (senescence)
    IF ( ALLOCATED (when_growthinit)) DEALLOCATE (when_growthinit)
    IF ( ALLOCATED (age))  DEALLOCATE (age)
    IF ( ALLOCATED (resp_hetero)) DEALLOCATE (resp_hetero)
    IF ( ALLOCATED (resp_maint)) DEALLOCATE (resp_maint)
    IF ( ALLOCATED (resp_growth)) DEALLOCATE (resp_growth)
    IF ( ALLOCATED (co2_fire)) DEALLOCATE (co2_fire)
    IF ( ALLOCATED (co2_to_bm_dgvm)) DEALLOCATE (co2_to_bm_dgvm)
    IF ( ALLOCATED (veget_lastlight)) DEALLOCATE (veget_lastlight)
    IF ( ALLOCATED (everywhere)) DEALLOCATE (everywhere)
    IF ( ALLOCATED (need_adjacent)) DEALLOCATE (need_adjacent)
    IF ( ALLOCATED (leaf_age)) DEALLOCATE (leaf_age)
    IF ( ALLOCATED (leaf_frac)) DEALLOCATE (leaf_frac)
    IF ( ALLOCATED (RIP_time)) DEALLOCATE (RIP_time)
    IF ( ALLOCATED (time_lowgpp)) DEALLOCATE (time_lowgpp)
    IF ( ALLOCATED (time_hum_min)) DEALLOCATE (time_hum_min)
    IF ( ALLOCATED (hum_min_dormance)) DEALLOCATE (hum_min_dormance)
    IF ( ALLOCATED (fvm)) DEALLOCATE (fvm)
    IF ( ALLOCATED (fv)) DEALLOCATE (fv)
    IF ( ALLOCATED (litterpart)) DEALLOCATE (litterpart)
    IF ( ALLOCATED (litter)) DEALLOCATE (litter)
    IF ( ALLOCATED (dead_leaves)) DEALLOCATE (dead_leaves)
    IF ( ALLOCATED (carbon)) DEALLOCATE (carbon)
    IF ( ALLOCATED (black_carbon)) DEALLOCATE (black_carbon)
    IF ( ALLOCATED (lignin_struc)) DEALLOCATE (lignin_struc)
   IF ( ALLOCATED (turnover_time)) DEALLOCATE (turnover_time)
   IF ( ALLOCATED (co2_flux_daily)) DEALLOCATE (co2_flux_daily)

    IF ( ALLOCATED (herbivores)) DEALLOCATE (herbivores)

    IF ( ALLOCATED (hori_index)) DEALLOCATE (hori_index)
    IF ( ALLOCATED (horipft_index)) DEALLOCATE (horipft_index)
    IF ( ALLOCATED (clay_fm)) DEALLOCATE (clay_fm)
    IF ( ALLOCATED (humrel_daily_x_fm)) DEALLOCATE (humrel_daily_x_fm)
    IF ( ALLOCATED (litterhum_daily_fm))  DEALLOCATE (litterhum_daily_fm) 
    IF ( ALLOCATED (t2m_daily_fm))  DEALLOCATE (t2m_daily_fm) 
    IF ( ALLOCATED (t2m_min_daily_fm))  DEALLOCATE (t2m_min_daily_fm)
    IF ( ALLOCATED (tsurf_daily_fm)) DEALLOCATE (tsurf_daily_fm)
    IF ( ALLOCATED (tsoil_daily_fm)) DEALLOCATE (tsoil_daily_fm)
    IF ( ALLOCATED (soilhum_daily_fm))  DEALLOCATE (soilhum_daily_fm) 
    IF ( ALLOCATED (precip_fm)) DEALLOCATE (precip_fm)
    IF ( ALLOCATED (gpp_daily_x_fm))  DEALLOCATE (gpp_daily_x_fm) 
    IF ( ALLOCATED (veget_x_fm)) DEALLOCATE (veget_x_fm)
    IF ( ALLOCATED (veget_max_x_fm)) DEALLOCATE (veget_max_x_fm)
    IF ( ALLOCATED (lai_x_fm))  DEALLOCATE (lai_x_fm) 
    IF ( ALLOCATED( isf )) DEALLOCATE( isf )
    IF ( ALLOCATED( nf_written )) DEALLOCATE( nf_written )
    IF ( ALLOCATED( nf_cumul )) DEALLOCATE( nf_cumul )
    IF ( ALLOCATED( times )) DEALLOCATE( times )
    IF ( ALLOCATED( nforce )) DEALLOCATE( nforce )
    IF ( ALLOCATED( control_moist )) DEALLOCATE( control_moist )
    IF ( ALLOCATED( control_temp )) DEALLOCATE( control_temp )
    IF ( ALLOCATED( soilcarbon_input )) DEALLOCATE( soilcarbon_input )
    

    ! 2. reset l_first
    l_first_stomate=.TRUE.
    ! 3. call to clear functions
    CALL get_reftemp_clear
    CALL season_clear
    CALL stomatelpj_clear
    CALL littercalc_clear
    CALL vmax_clear
  END SUBROUTINE stomate_clear

  SUBROUTINE stomate_var_init (kjpindex, qsintcst, &
                                veget, veget_max, leaf_age, leaf_frac, tlong_ref, t2m_month, dead_leaves, &
                                veget_x, lai_x, qsintmax, deadleaf_cover, assim_param_x)

    ! this subroutine outputs values of assim_param etc. only if ok_stomate = .TRUE.
    ! otherwise,the calling procedure must do it itself.

    ! interface description
    ! input scalar
    ! Domain size
    INTEGER(long), INTENT (in)                                 :: kjpindex
    ! Coefficient for interception reservoir
    REAL(stnd), INTENT(in)                                     :: qsintcst
    ! input fields
    ! fractional coverage: actually covered space
    REAL(stnd), DIMENSION(kjpindex, npft), INTENT(in)          :: veget
    ! fractional coverage: maximum covered space
    REAL(stnd), DIMENSION(kjpindex, npft), INTENT(in)          :: veget_max
    ! "long term" 2 meter reference temperatures (K)
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)                :: tlong_ref
    ! "monthly" 2 meter temperatures (K)
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)                :: t2m_month
    ! dead leaves on ground, per PFT, metabolic and structural, in gC/(m**2 of nat/agri ground)
    REAL(stnd), DIMENSION(kjpindex,npft,nlitt), INTENT(in)     :: dead_leaves
    ! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)           :: veget_x
    ! Surface foliere
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)           :: lai_x
    ! modified fields (actually NOT modified)
    ! leaf age (d)
    REAL(stnd),DIMENSION(kjpindex,npft,nleafages),INTENT(inout):: leaf_age
    ! fraction of leaves in leaf age class
    REAL(stnd),DIMENSION(kjpindex,npft,nleafages),INTENT(inout):: leaf_frac
    ! output scalar
    ! output fields
    ! Maximum water on vegetation for interception
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)          :: qsintmax
    ! fraction of soil covered by dead leaves
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)              :: deadleaf_cover
    ! min+max+opt temps & vmax for photosynthesis
    REAL(stnd), DIMENSION (kjpindex,nvm,npco2), INTENT (out)   :: assim_param_x

    ! local declaration
    ! dummy time step, must be zero
    REAL(stnd), PARAMETER                                      :: dt_0 = 0.
    REAL(stnd), DIMENSION (kjpindex,nvm)                       :: vcmax
    REAL(stnd), DIMENSION (kjpindex,nvm)                       :: vjmax
    ! Min temperature for photosynthesis (deg C)
    REAL(stnd), DIMENSION(kjpindex,npft)                       :: t_photo_min
    ! Opt temperature for photosynthesis (deg C)
    REAL(stnd), DIMENSION(kjpindex,npft)                       :: t_photo_opt
    ! Max temperature for photosynthesis (deg C)
    REAL(stnd), DIMENSION(kjpindex,npft)                       :: t_photo_max
    ! Fraction of space that is natural
    REAL(stnd), DIMENSION(kjpindex)                            :: space_nat
    ! Index
    INTEGER(short)                                             :: j

    IF ( control%ok_stomate ) THEN

      !
      ! 1 photosynthesis parameters
      !

      !
      ! 1.1 vcmax
      !     only if STOMATE is activated
      !

      CALL vmax (kjpindex, dt_0, &
                 leaf_age, leaf_frac, &
                 vcmax, vjmax)

      !
      ! 1.2 assimilation temperatures
      !

      CALL assim_temp(kjpindex, tlong_ref, t2m_month, &
                      t_photo_min, t_photo_opt, t_photo_max)

      !
      ! 1.3 transform into nvm vegetation types
      !

      CALL stomate_var_xout (kjpindex,vcmax,veget,zero,assim_param_x(:,:,ivcmax))
      CALL stomate_var_xout (kjpindex,vjmax,veget,zero,assim_param_x(:,:,ivjmax))

      CALL stomate_var_xout (kjpindex,t_photo_min,veget,zero,assim_param_x(:,:,itmin))
      CALL stomate_var_xout (kjpindex,t_photo_opt,veget,zero,assim_param_x(:,:,itopt))
      CALL stomate_var_xout (kjpindex,t_photo_max,veget,zero,assim_param_x(:,:,itmax))

      !
      ! 2 dead leaf cover
      !

      ! first recalculate fraction of natural space

      space_nat(:) = 1.0

      DO j = 1, npft
        IF ( .NOT. natural(j) ) THEN
          space_nat(:) = space_nat(:) - veget_max(:,j)
        ENDIF
      ENDDO

      CALL deadleaf (kjpindex, space_nat, dead_leaves, deadleaf_cover)

      !
      ! 3 qsintmax
      !

      qsintmax(:,:) = qsintcst * veget_x(:,:) * lai_x(:,:)

    ENDIF ! ok_stomate = .TRUE.

  END SUBROUTINE stomate_var_init

  SUBROUTINE stomate_accu (npts, n_dim2, dt_tot, dt, ldmean, field_in, field_out)

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                 :: npts
    ! 2nd dimension (1 or npft)
    INTEGER, INTENT(in)                                 :: n_dim2
    ! Time step of STOMATE (days)
    REAL, INTENT(in)                                    :: dt_tot
    ! Time step in days
    REAL(stnd), INTENT(in)                              :: dt
    ! Calculate mean ?
    LOGICAL, INTENT(in)                                 :: ldmean
    ! Daily field
    REAL(stnd), DIMENSION(npts,n_dim2), INTENT(in)      :: field_in

    ! 0.2 modified field

    ! Annual field
    REAL(stnd), DIMENSION(npts,n_dim2), INTENT(inout)   :: field_out

    ! =========================================================================

    !
    ! 1 accumulation
    !

    field_out(:,:) = field_out(:,:) + field_in(:,:) * dt

    !
    ! 2 mean fields
    !

    IF (ldmean) THEN

      field_out(:,:) = field_out(:,:) / dt_tot

    ENDIF

  END SUBROUTINE stomate_accu

  SUBROUTINE stomate_vegconvert (kjpindex,which_way,fraction_nobio, &
                                   veget_max_x,veget_x,veget_max,veget,fvm,fv)

    !
    ! 0 declarations
    !

    !
    ! 0.1 input
    !
    ! 0.1.1 input scalar
    !
    ! Domain size
    INTEGER(long),INTENT (in)                          :: kjpindex
    ! veget -> veget_x ('out') or veget_x -> veget ('in') ?
    CHARACTER(LEN=*), INTENT(in)                       :: which_way
    ! Fraction of land covered by lakes, land ice, cities, ...
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)        :: fraction_nobio
    !
    ! 0.2 modified
    !
    ! Max vegetation fraction of hydrological module
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(inout) :: veget_max_x
    ! Vegetation fraction of hydrological module
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(inout) :: veget_x
    ! Max vegetation fraction of STOMATE
    REAL(stnd), DIMENSION(kjpindex,npft), INTENT(inout) :: veget_max
    ! Vegetation fraction of STOMATE
    REAL(stnd), DIMENSION(kjpindex,npft), INTENT(inout) :: veget
    ! factor that converts veget_max_x into veget_max
    REAL(stnd), DIMENSION(kjpindex,npft), INTENT(inout) :: fvm
    ! factor that converts veget_x into veget
    REAL(stnd), DIMENSION(kjpindex,npft), INTENT(inout) :: fv
    !
    ! 0.3 local
    !
    INTEGER(long)                                      :: j,jv

    IF ( which_way .EQ. 'in' ) THEN

      !
      ! 1 Convert veget_x into veget
      !

      DO j = 1, npft
        !
        WHERE ( (1.-fraction_nobio(:)) .GT. 0.0 )
          !
          veget(:,j) = veget_x(:,ipft_sechiba(j)) * fv(:,j) / ( 1.-fraction_nobio(:) )
          veget_max(:,j) = veget_max_x(:,ipft_sechiba(j)) * fvm(:,j) / ( 1.-fraction_nobio(:) )
          !
        ELSEWHERE
          !
          veget(:,j) = 0.
          veget_max(:,j) = 0.
          !
        ENDWHERE
        !
      ENDDO

    ELSEIF ( which_way .EQ. 'out' ) THEN

      !
      ! 2 Convert veget into veget_x and calculate fv and fvm for next time step
      !

      !
      ! 2.1 vegetation
      !

      veget_x(:,:) = 0.0 
      veget_max_x(:,:) = 0.0 

      DO j = 1, npft
        !
        veget_x(:,ipft_sechiba(j)) = &
           veget_x(:,ipft_sechiba(j)) + veget(:,j) * ( 1.-fraction_nobio(:) )
        veget_max_x(:,ipft_sechiba(j)) = &
           veget_max_x(:,ipft_sechiba(j)) + veget_max(:,j) * ( 1.-fraction_nobio(:) )
        !
      ENDDO

      !
      ! 2.2 bare soil
      !     for bare soil, veget_max is actually the potential minimum, as it is
      !       calculated using the potential maximum vegetation cover.
      !     veget_max has no real physical meaning anyway.
      !

      veget_max_x(:,ibare_sechiba) = 1.-fraction_nobio(:)
      veget_x(:,ibare_sechiba) = 1.-fraction_nobio(:)

      DO jv = 1, nvm
        !
        IF ( jv .NE. ibare_sechiba ) THEN
          !
          veget_max_x(:,ibare_sechiba) = veget_max_x(:,ibare_sechiba) - veget_max_x(:,jv)
          veget_x(:,ibare_sechiba) = veget_x(:,ibare_sechiba) - veget_x(:,jv)
          !
        ENDIF
        !
      ENDDO

      ! potential minimum bare soil cover cannot be less than 0
      veget_max_x(:,ibare_sechiba) = MAX( veget_max_x(:,ibare_sechiba), 0._stnd )

      !
      ! 2.3 calculate fv and fvm
      !

      DO j = 1, npft
        !
        WHERE ( veget_max_x(:,ipft_sechiba(j)) .GT. 0. )
          fvm(:,j) = veget_max(:,j)/veget_max_x(:,ipft_sechiba(j)) * ( 1.-fraction_nobio(:) )
        ELSEWHERE
          fvm(:,j) = 0.0
        ENDWHERE
        !
        WHERE ( veget_x(:,ipft_sechiba(j)) .GT. 0. )
          fv(:,j) = veget(:,j)/veget_x(:,ipft_sechiba(j)) * ( 1.-fraction_nobio(:) )
        ELSEWHERE
          fv(:,j) = 0.0
        ENDWHERE
        !
      ENDDO

    ELSE

      WRITE(numout,*) 'stomate_vegconvert: which_way = ', which_way
      STOP 'Cannot go this way.'

    ENDIF

  END SUBROUTINE stomate_vegconvert

  SUBROUTINE stomate_var_xout (kjpindex,var,vfrac,bare_val,var_x)

    ! this subroutine outputs a variable (e.g. lai_x) on nvm vegetation types
    !   given an input (e.g. lai) on npft vegetation types. Ponderation is done 
    !   using vfrac (= veget or veget_max)

    !
    ! 0 declarations
    !

    !
    ! 0.1 input
    !
    ! 0.1.1 input scalar
    !
    ! Domain size
    INTEGER(long),INTENT (in)                          :: kjpindex
    ! variable defined on npft
    REAL(stnd), DIMENSION(kjpindex,npft), INTENT(in)   :: var
    ! vegetation fraction
    REAL(stnd), DIMENSION(kjpindex,npft), INTENT(in)   :: vfrac
    ! value for bare ground
    REAL(stnd), INTENT(in)                             :: bare_val
    !
    ! 0.2 output
    ! variable defined on nvm
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(out)   :: var_x
    !
    ! 0.3 local
    !
    INTEGER(long)                                      :: j
    REAL(stnd), DIMENSION(kjpindex,nvm)                :: vfracsum

    !
    ! 1 Calculate ponderated value for vegetation types
    !

    var_x(:,:) = 0.0
    vfracsum(:,:) = 0.0

    DO j = 1, npft
      var_x(:,ipft_sechiba(j)) = var_x(:,ipft_sechiba(j)) + var(:,j) * vfrac(:,j)
      vfracsum(:,ipft_sechiba(j)) = vfracsum(:,ipft_sechiba(j)) + vfrac(:,j)
    ENDDO

    WHERE ( vfracsum(:,:) .GT. 0.0 )
      var_x(:,:) = var_x(:,:) / vfracsum(:,:)
    ENDWHERE

    !
    ! 2 impose a value for bare soil
    !

    var_x(:,ibare_sechiba) = bare_val

  END SUBROUTINE stomate_var_xout

  SUBROUTINE init_forcing (kjpindex,nsfm,nsft)

    INTEGER(long), INTENT(in)                     :: kjpindex
    INTEGER(long), INTENT(in)                     :: nsfm
    INTEGER(long), INTENT(in)                     :: nsft

    LOGICAL                                       :: l_error
    INTEGER(long)                                 :: ier

    l_error = .FALSE.

    ALLOCATE (clay_fm(kjpindex,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (humrel_daily_x_fm(kjpindex,nvm,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (litterhum_daily_fm(kjpindex,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_daily_fm(kjpindex,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (t2m_min_daily_fm(kjpindex,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (tsurf_daily_fm(kjpindex,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (tsoil_daily_fm(kjpindex,nbdl,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (soilhum_daily_fm(kjpindex,nbdl,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (precip_fm(kjpindex,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (gpp_daily_x_fm(kjpindex,nvm,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (veget_x_fm(kjpindex,nvm,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (veget_max_x_fm(kjpindex,nvm,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE (lai_x_fm(kjpindex,nvm,nsfm), stat=ier) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE( isf(nsfm) ) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE( nf_written(nsft) ) ; l_error = l_error .OR. (ier.NE.0)
    ALLOCATE( nf_cumul(nsft) ) ; l_error = l_error .OR. (ier.NE.0)

    IF ( l_error ) THEN
      STOP 'Problem with memory allocation: forcing variables'
    ENDIF

    CALL forcing_zero

  END SUBROUTINE init_forcing

  SUBROUTINE forcing_zero

    clay_fm(:,:) = 0.0
    humrel_daily_x_fm(:,:,:) = 0.0
    litterhum_daily_fm(:,:) = 0.0
    t2m_daily_fm(:,:) = 0.0
    t2m_min_daily_fm(:,:) = 0.0
    tsurf_daily_fm(:,:) = 0.0
    tsoil_daily_fm(:,:,:) = 0.0
    soilhum_daily_fm(:,:,:) = 0.0
    precip_fm(:,:) = 0.0
    gpp_daily_x_fm(:,:,:) = 0.0
    veget_x_fm(:,:,:) = 0.0
    veget_max_x_fm(:,:,:) = 0.0
    lai_x_fm(:,:,:) = 0.0

  END SUBROUTINE forcing_zero

  SUBROUTINE forcing_write(forcing_id,ibeg,iend)

    INTEGER(long), INTENT(in)                     :: forcing_id
    INTEGER(long), INTENT(in)                     :: ibeg, iend

    INTEGER(long)                                 :: iisf, iblocks, nblocks
    INTEGER(long)                                 :: ier
    INTEGER(long), DIMENSION(0:2)                 :: ifirst, ilast
    INTEGER, PARAMETER                            :: ndm = 10
    INTEGER, DIMENSION(ndm)                       :: start, count
    INTEGER                                       :: ndim, vid

    !
    ! determine blocks of forcing states that are contiguous in memory
    !
    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    !
    DO iisf = ibeg, iend
      !
      IF ( ( nblocks .NE. 0 ) .AND. &
           ( isf(iisf) .EQ. isf(ilast(nblocks)) + 1 ) ) THEN
        !
        ! element is contiguous with last element found
        !
        ilast(nblocks) = iisf
        !
      ELSE
        !
        ! found first element of new block
        !
        nblocks = nblocks + 1
        IF ( nblocks .GT. 2 ) STOP 'Problem in forcing_write'
        !
        ifirst(nblocks) = iisf
        ilast(nblocks) = iisf
        !
      ENDIF
      !
    ENDDO

    DO iblocks = 1, nblocks
      !
      IF ( ifirst(iblocks) .NE. ilast(iblocks) ) THEN
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( clay_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'clay', start, count, &
                                clay_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( humrel_daily_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'humrel', start, count, &
                                humrel_daily_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( litterhum_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'litterhum', start, count, &
                                litterhum_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( t2m_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 't2m', start, count, &
                                t2m_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( t2m_min_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 't2m_min', start, count, &
                                t2m_min_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( tsurf_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'tsurf', start, count, &
                                tsurf_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( tsoil_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'tsoil', start, count, &
                                tsoil_daily_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( soilhum_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'soilhum', start, count, &
                                soilhum_daily_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( precip_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'precip', start, count, &
                                precip_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( gpp_daily_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'gpp', start, count, &
                                gpp_daily_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( veget_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'veget', start, count, &
                                veget_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( veget_max_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'veget_max', start, count, &
                                veget_max_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( lai_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_PUT_VARA_STND( forcing_id, 'lai', start, count, &
                                lai_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
      ENDIF
      !
    ENDDO

    nf_written(isf(:)) = .TRUE.

  END SUBROUTINE forcing_write

  SUBROUTINE forcing_read(forcing_id,nsfm)
 
    INTEGER(long), INTENT(in)                     :: forcing_id
    INTEGER(long), INTENT(in)                     :: nsfm
 
    INTEGER(long)                                 :: iisf, iblocks, nblocks
    INTEGER(long)                                 :: ier
    INTEGER(long), DIMENSION(0:2)                 :: ifirst, ilast
    INTEGER, PARAMETER                            :: ndm = 10
    INTEGER, DIMENSION(ndm)                       :: start, count
    INTEGER                                       :: ndim, vid

    ! set to zero if the corresponding forcing state has not yet been written
    ! into the file
    !
    DO iisf = 1, nsfm
      !
      IF ( .NOT. nf_written(isf(iisf)) ) THEN
        ! 
        clay_fm(:,iisf) = 0.0
        humrel_daily_x_fm(:,:,iisf) = 0.0
        litterhum_daily_fm(:,iisf) = 0.0
        t2m_daily_fm(:,iisf) = 0.0
        t2m_min_daily_fm(:,iisf) = 0.0
        tsurf_daily_fm(:,iisf) = 0.0
        tsoil_daily_fm(:,:,iisf) = 0.0
        soilhum_daily_fm(:,:,iisf) = 0.0
        precip_fm(:,iisf) = 0.0
        gpp_daily_x_fm(:,:,iisf) = 0.0
        veget_x_fm(:,:,iisf) = 0.0
        veget_max_x_fm(:,:,iisf) = 0.0
        lai_x_fm(:,:,iisf) = 0.0
        ! 
      ENDIF
      !
    ENDDO
    !
    ! determine blocks of forcing states that are contiguous in memory
    !
    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    !
    DO iisf = 1, nsfm
      !
      IF ( nf_written(isf(iisf)) ) THEN
        !
        IF ( ( nblocks .NE. 0 ) .AND. &
             ( isf(iisf) .EQ. isf(ilast(nblocks)) + 1 ) ) THEN
          !
          ! element is contiguous with last element found
          !
          ilast(nblocks) = iisf
          !
        ELSE
          !
          ! found first element of new block
          !
          nblocks = nblocks + 1
          IF ( nblocks .GT. 2 ) STOP 'Problem in forcing_read'
          !
          ifirst(nblocks) = iisf
          ilast(nblocks) = iisf
          !
        ENDIF
        !
      ENDIF
      !
    ENDDO

    DO iblocks = 1, nblocks
      !
      IF ( ifirst(iblocks) .NE. ilast(iblocks) ) THEN
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( clay_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'clay', start, count, &
                                clay_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( humrel_daily_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'humrel', start, count, &
                                humrel_daily_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( litterhum_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'litterhum', start, count, &
                                litterhum_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( t2m_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 't2m', start, count, &
                                t2m_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( t2m_min_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 't2m_min', start, count, &
                                t2m_min_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( tsurf_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'tsurf', start, count, &
                                tsurf_daily_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( tsoil_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'tsoil', start, count, &
                                tsoil_daily_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( soilhum_daily_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'soilhum', start, count, &
                                soilhum_daily_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 2; start(:) = 1; count(1:ndim) = SHAPE( precip_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'precip', start, count, &
                                precip_fm(:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( gpp_daily_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'gpp', start, count, &
                                gpp_daily_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( veget_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'veget', start, count, &
                                veget_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( veget_max_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'veget_max', start, count, &
                                veget_max_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
        ndim = 3; start(:) = 1; count(1:ndim) = SHAPE( lai_x_fm )
        start(ndim) = isf(ifirst(iblocks)) ; count(ndim) = isf(ilast(iblocks)) - isf(ifirst(iblocks)) + 1
        ier = NF_GET_VARA_STND( forcing_id, 'lai', start, count, &
                                lai_x_fm(:,:,ifirst(iblocks):ilast(iblocks)) )
        !
      ENDIF
      !
    ENDDO
 
  END SUBROUTINE forcing_read

  SUBROUTINE setlai(npts,lai)

    ! routine to force the lai in STOMATE (for assimilation procedures)
    ! for the moment setlai only gives the lai from stomate, this routine must be written in the future

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                               :: npts

    ! 0.3 output

    REAL(stnd), DIMENSION(npts,npft), INTENT(out)                :: lai

    ! 0.4 local definitions
    INTEGER                                                      :: j
          DO j = 1, npft
             WHERE ( veget_max(:,j) .GT. 0.0 )
                lai(:,j) = biomass(:,j,ileaf) / veget_max(:,j) * sla(j)
             ELSEWHERE
                lai(:,j) = 0.0
             ENDWHERE
          ENDDO
     END SUBROUTINE setlai

  !! BEGINNVADD

  ! 
  ! ==============================================================================
  ! NV clowproc_soil permet de faire l'interface entre sechiba et stomate lorsque
  ! l'on appele uniquement le calcul du carbon sol 
  !

  SUBROUTINE stomate_soil (npts, dt, clay_r,space_nat_r,&
                            soilcarbon_input, control_temp, control_moist,resp_hetero_soil)

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                               :: npts
    ! time step in days
    REAL(stnd), INTENT(in)                                            :: dt
    ! quantity of carbon going into carbon pools from litter decomposition
    !   (gC/(m**2 of nat/agri ground)/day)
    REAL(stnd), DIMENSION(npts), INTENT(in)           :: clay_r
   REAL(stnd), DIMENSION(npts), INTENT(in)           :: space_nat_r

    REAL(stnd), DIMENSION(npts,ncarb,nvegtypes), INTENT(in)           :: soilcarbon_input
    ! temperature control of heterotrophic respiration
    REAL(stnd), DIMENSION(npts,nlevs), INTENT(in)                     :: control_temp
    ! moisture control of heterotrophic respiration
    REAL(stnd), DIMENSION(npts,nlevs), INTENT(in)                     :: control_moist

    ! 0.3 output

    ! soil heterotrophic respiration (first in gC/day/m**2 of natural/agricultural ground,
    !   but output in gC/day/m**2 of total ground)
    REAL(stnd), DIMENSION(npts,nvegtypes), INTENT(out)                :: resp_hetero_soil

    write_carbonforce = .FALSE.

    CALL soilcarbon (npts, dt, clay_r, space_nat_r, &
                     soilcarbon_input, control_temp, control_moist, &
                     carbon, &
                     resp_hetero_soil)

  END SUBROUTINE stomate_soil

  SUBROUTINE pondere_nat (npts,pondere)
    INTEGER, INTENT(in)                                               :: npts
    REAL(stnd), DIMENSION(npts,npft), INTENT(out)                     :: pondere
    INTEGER :: n

    do n=1,npft
       if (natural(n))  THEN
          pondere(:,n)=space_nat(:)
       else
          pondere(:,n)=1.-space_nat(:)
      endif
    enddo
   END SUBROUTINE pondere_nat
  SUBROUTINE pondere_nat_vegfrac (npts,pondere)
    INTEGER, INTENT(in)                                               :: npts
    REAL(stnd), DIMENSION(npts,npft), INTENT(out)                     :: pondere
    INTEGER :: n

    do n=1,npft
      if (natural(n))  then
         pondere(:,n)=space_nat(:)*veget_max(:,n)
      else
         pondere(:,n)=(1.-space_nat(:))*veget_max(:,n)
      end if
    enddo
  END SUBROUTINE pondere_nat_vegfrac
  SUBROUTINE pondere_vegfrac (npts,pondere)
    INTEGER, INTENT(in)                                               :: npts
    REAL(stnd), DIMENSION(npts,npft), INTENT(out)                     :: pondere
    INTEGER :: n

    do n=1,npft
       pondere(:,n)=veget_max(:,n)
    enddo
  END SUBROUTINE pondere_vegfrac
!! ENDNVADD

END MODULE stomate
